#include "params.h"
      subroutine srftsb(isrfty  ,fnt     ,dfntdt  ,sndpth  ,tsbsf   )
C-----------------------------------------------------------------------
C
C Computes surface/sub surface temperature change assuming thermal
C diffusion; solved by backward implicit time stepping procedure with
C linearized top flux boundary condition and specified bottom flux
C boundary condition.
C
C Assumes 7 surface types, along with their geometric and thermal
C properties, as follows:
C
C   1  Ocean
C   2  Sea ice
C   3  Permanent land ice
C   4  Equatorial forest
C   5  Mid-latitude and high-latitude forest
C   6  Shrubland or tundra
C   7  Desert
C
C No change is made in the ocean surface temperature.
C
C Assumes plevmx number of surface/sub-surface layers, and ascribes
C physical properties for these layers for each of the 6 surface types
C above (numbers 2 to 7).
C
C Over sea-ice, heat flux into bottom layer is allowed; over all other
C (land) surfaces, a zero bottom flux boundary condition is imposed.
C
C Sub-surface layers are indexed 1 at the surface, increasing downwards
C to plevmx.  Layers have mid-points and interfaces between layers.
C Temperatures are defined at mid-points, while fluxes between layers
C and the top/bottom media are defined at layer interfaces.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Briegleb
C Standardized:      L. Buja, June 1992
C Reviewed:          B. Briegleb, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "comtsc.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     isrfty(plond)       ! Surface type index (1..7)
      real fnt(plond),         ! Top surface/atmosphere net energy flux
     $     dfntdt(plond),      ! ts partial derivative of net sfc flux
     $     sndpth(plond)       ! Snow depth (liquid water equivalent)
C
C Input/output arguments
C
      real tsbsf(plond,plevmx) ! Initial/final surface/sub-surface tmps
C
C---------------------------Local parameters----------------------------
C
      integer    psrfty        ! Number of surface types
      parameter (psrfty=7)     
C
C---------------------------Local variables-----------------------------
C
      integer i,               ! Longitude index
     $        ier,             ! Error return on tridiagonal solver
     $        indx,            ! Surface type index
     $        n                ! Sub-surface layer index
C
      real cmass(plond,plevmx),! Specific heat of soil (J/kg/K)
     $     rho(plond,plevmx),  ! Mass densty of sub-sfc mat (kg/m3)
     $     tk(plond,plevmx),   ! Thermal conductivity (watts/m/K)
     $     z(plond,0:plevmx)   ! Interface geometrical depth (m)
C
C Specified (and invariant) thermal properties for surface types
C
      real cmtype(psrfty,plevmx),  ! Mass specific heat (J/kg/K)
     $     rhtype(psrfty,plevmx),  ! Mass density (kg/m3)
     $     thckly(psrfty,plevmx),  ! Layer thicknesses (m)
     $     tktype(psrfty,plevmx)   ! Thermal conductivity (J/m/s)
      save cmtype,rhtype,thckly,tktype
C
      data cmtype /4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2,
     $             4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2,
     $             4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2,
     $             4.20e3,2.07e3,2.07e3,1.04e3,7.20e2,5.60e2,4.16e2/
C
      data rhtype /1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3,
     $             1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3,
     $             1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3,
     $             1.00e3,9.20e2,9.20e2,2.50e3,2.50e3,2.50e3,2.50e3/
C
      data thckly / 2., .500, .250, .050, .090, .080, .120,
     $              5., .500, .500, .366, .390, .435, .492,
     $             10., .500, .500,1.369,1.459,1.628,1.841,
     $             33.,.500,8.500,6.990,7.450,8.310,9.400/
C
      data tktype /15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 ,
     $             15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 ,
     $             15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 ,
     $             15.0 ,2.200 ,2.200 ,1.408 ,1.104 ,1.071 ,1.019 /
C
C Properties of ice and air used to determine snow thermal properties
C
      real cmair,     ! Mass specific heat of air
     $     cmice,     ! Mass specific heat of ice
     $     frcair,    ! Fraction of air assumed in mix of ice and air
     $     rhair,     ! Mass density of surface air
     $     rhice,     ! Mass density of ice
     $     tkair,     ! Thermal conductivity of air
     $     tkice      ! Thermal conductivity of ice
      save cmair,cmice,frcair,rhair,rhice,tkair,tkice
C
      data cmair  /1.00e3/
      data cmice  /2.07e3/
      data frcair /0.90/
      data rhair  /1.25 /
      data rhice  /9.20e2/
      data tkair  /0.025/
      data tkice  /2.200/
C
      real diag(plond,plevmx),    ! Diagonal matrix elements
     $     htsrc(plond,plevmx),   ! External heat source (W/m3)
     $     rhs(plond,plevmx),     ! Rhs of tri-diagonal matrix equation
     $     sbdiag(plond,plevmx),  ! Sub-diagonal matrix elements
     $     spdiag(plond,plevmx),  ! Super-diagonal matrix elements
     $     tin(plond,plevmx),     ! Initial sub-surface temperatures
     $     ws(plond*plevmx)       ! Working storage for mtdlss
C
      real cmsnow,        ! Snow mass heat capacity
     $     cmty(plond),   ! Layer mass heat capacity
     $     crt,           ! cmass*rho*rdtime
     $     delz,          ! Layer thickness
     $     delzmn,        ! Thick from mid-point to lyr above mid-point
     $     delzpl,        ! Thick from mid-point to lyr below mid-point
     $     fbt(plond),    ! Ocean heat flux into sea-ice
     $     fmns,          ! 1/(delz*delzmn)
     $     fpls,          ! 1/(delz*delzpl)
     $     msnow,         ! Mass path of snow
     $     msoil,         ! Mass path of soil
     $     rdtime,        ! Inverse model time step
     $     rhsnow,        ! Snow mass density
     $     rhty(plond),   ! Layer mass density
     $     rztop          ! 1/ztop
      real thck(plond),   ! Layer thickness
     $     tkbot,         ! Bottom layer top interf thermal conduct
     $     tkmns,         ! Layer bottom interface thermal conduct
     $     tkpls,         ! Layer top interface thermal conductivity
     $     tksnow,        ! Snow thermal conducitivity
     $     tktop,         ! Top layer bottom interface thermal conduct
     $     tkty(plond),   ! Layer thermal conductivity
     $     tmp,           ! crt - dfntdt(i)*rztop
     $     zbot,          ! Bottom layer thickness
     $     zm,            ! Present layer mid-point depth
     $     zmmn,          ! Layer above mid-point depth
     $     zmpl,          ! Layer below mid-point depth
     $     zsnow,         ! Snow geometric depth
     $     ztop           ! Top layer thickness
C
      logical scvr(plond) ! True if surface snow covered
      external mtdlss, endrun
C
C-----------------------------------------------------------------------
C
      rdtime = 1./dtime
      cmsnow = (1.-frcair)*cmice + frcair*cmair
      rhsnow = (1.-frcair)*rhice + frcair*rhair
      tksnow = (1.-frcair)*tkice + frcair*tkair
C
C No external heat source currently:
C
      do n=1,plevmx
         do i=1,plon
            htsrc(i,n) = 0.0
         end do
      end do
C
C Define logical for snow covered surfaces:
C
      do i=1,plon
         scvr(i) = sndpth(i).gt.0.
      end do
C
C Define thermal properities for each sub/surface layer, starting
C with the top layer
C
      do i=1,plon
         indx    = isrfty(i)
         thck(i) = thckly(indx,1)
         cmty(i) = cmtype(indx,1)
         rhty(i) = rhtype(indx,1)
         tkty(i) = tktype(indx,1)
      end do
C
      do i=1,plon
C
C Initialize fields for no snow cover
C
         z(i,0)     = 0.0
         z(i,1)     = thck(i)
         cmass(i,1) = cmty(i)
         rho(i,1)   = rhty(i)
         tk(i,1)    = tkty(i)
C
C Modify layer 1 fields for snow cover if present
C
         if(scvr(i)) then
C
C Snow equivlnt depth times snow liquid water depth gives the physical
C depth of snow for thermal conduction computation; snow is mixed
C uniformly by mass with the top surface layer
C
           zsnow     = sndpth(i)*snwedp
           msnow     = rhsnow*zsnow
           msoil     = rhty(i)*thck(i)
           rho(i,1)  = (msnow*rhsnow + msoil*rhty(i))/(msnow+msoil)
           cmass(i,1)= (msnow*cmsnow + msoil*cmty(i))/(msnow+msoil)
           tk(i,1)   = (msnow*tksnow + msoil*tkty(i))/(msnow+msoil)
           z(i,1)    = (msnow+msoil) / rho(i,1)
         end if
C
      end do
C
C Set surface thermal properties for the lower sub/surface layers:
C
      do n=2,plevmx
         do i=1,plon
            indx       = isrfty(i)
            thck(i)    = thckly(indx,n)
            cmass(i,n) = cmtype(indx,n)
            rho(i,n)   = rhtype(indx,n)
            tk(i,n)    = tktype(indx,n)
         end do
         do i=1,plon
            z(i,n)  = z(i,n-1) + thck(i)
         end do
      end do
C
C Define set of linear equations for temperature
C
      do n=1,plevmx
         do i=1,plon
            tin(i,n) = tsbsf(i,n)
         end do
      end do
C
C If sea ice, compute heat flux from underlying ocean, assumed to be at
C the temperature of -2C
C
      do i=1,plon
         fbt(i) = 0.0
         if(isrfty(i).eq.2) then
            zbot    = 0.5*(z(i,plevmx) - z(i,plevmx-1))
            fbt(i)  = -tk(i,plevmx)*(271.16 - tin(i,plevmx))/zbot
         end if
      end do
C
C Set up linear equations
C
      do i=1,plon
         sbdiag(i,1)      = 0.
         spdiag(i,plevmx) = 0.
      end do
C
C Single layer
C
      if(plevmx.eq.1) then
         do i=1,plon
            rztop     = 1./(z(i,1) - z(i,0))
            crt       = (cmass(i,1)*rho(i,1)*rdtime)
            diag(i,1) = crt - dfntdt(i)*rztop
            rhs(i,1)  = diag(i,1)*tin(i,1) + fnt(i)*rztop -
     $                  fbt(i)*rztop + htsrc(i,1)
         end do
C
C More than one layer
C
      else if(plevmx.gt.1) then
C
C Top layer
C
         do i=1,plon
            crt         = cmass(i,1)*rho(i,1)*rdtime
            ztop        = z(i,1) - z(i,0)
            rztop       = 1./ztop
            tktop       = 0.5*(tk(i,1) + tk(i,2))
            zmpl        = 0.5*(z(i,2) + z(i,1))
            zm          = 0.5*(z(i,1) + z(i,0))
            delzpl      = zmpl - zm
            fpls        = 1./(ztop*delzpl)
            tmp         = crt - dfntdt(i)*rztop
            diag(i,1)   = tmp + tktop*fpls
            spdiag(i,1) = -tktop*fpls
            rhs(i,1)    = tmp*tin(i,1) + fnt(i)*rztop + htsrc(i,1)
         end do
C
C Intermediate layers
C
         do n=2,plevmx-1
            do i=1,plon
               crt         = cmass(i,n)*rho(i,n)*rdtime
               delz        = z(i,n) - z(i,n-1)
               zmpl        = 0.5*(z(i,n+1) + z(i,n))
               zm          = 0.5*(z(i,n)   + z(i,n-1))
               zmmn        = 0.5*(z(i,n-1) + z(i,n-2))
               delzpl      = zmpl - zm
               delzmn      = zm - zmmn
               fpls        = 1./(delz*delzpl)
               fmns        = 1./(delz*delzmn)
               tkpls       = 0.5*(tk(i,n+1)+tk(i,n))
               tkmns       = 0.5*(tk(i,n)+tk(i,n-1))
               sbdiag(i,n) = -tkmns*fmns
               diag(i,n)   = crt + (tkpls*fpls + tkmns*fmns)
               spdiag(i,n) = -tkpls*fpls
               rhs(i,n)    = crt*tin(i,n) + htsrc(i,n)
            end do
         end do
C
C Bottom layer
C
         do i=1,plon
            crt        = cmass(i,plevmx)*rho(i,plevmx)*rdtime
            zbot       = z(i,plevmx) - z(i,plevmx-1)
            zm         = 0.5*(z(i,plevmx)   + z(i,plevmx-1))
            zmmn       = 0.5*(z(i,plevmx-1) + z(i,plevmx-2))
            delzmn     = zm - zmmn
            tkbot      = 0.5*(tk(i,plevmx-1) + tk(i,plevmx))
            fmns       = 1./(zbot*delzmn)
            sbdiag(i,plevmx) = -tkbot*fmns
            diag(i,plevmx)   = crt + (tkbot*fmns)
            rhs(i,plevmx)    = crt*tin(i,plevmx) - fbt(i)/zbot
     $                          + htsrc(i,plevmx)
         end do
      end if
C
C For the linear equation ax = b,  a and b arrays are now set;
C solve for temperatures (x):
C
      if(plevmx.eq.1) then
         do i=1,plon
            tsbsf(i,1) = rhs(i,1)/diag(i,1)
         end do
      else
         call mtdlss(sbdiag  ,diag   ,spdiag   ,rhs     ,tsbsf ,
     $               plevmx  ,plond  ,plon     ,ws      ,plond*plevmx ,
     $               ier     )
         if(ier.ne.0) then
           write(6,*) 'SRFTSB: Error returned from mtdlss'
           call blowup
         end if
      end if
C
      return
      end
