#include "params.h"
      subroutine phcs(pmn     ,hmn     ,ix      ,x1)
C-----------------------------------------------------------------------
C
C Compute associated Legendre functions of the first kind of order m and
C degree n, and the associated derivatives for arg x1.  The associated
C Legendre functions are evaluated using relationships contained in
C "Tables of Normalized Associated Legendre Polynomials",
C S. L. Belousov (1962).  Both the functions and their derivatives are
C ordered in a linear stored rectangular array (with a large enough
C domain to contain the particular wavenumber truncation defined in the
C pspect common block) by column. m = 0->ptrm, and  n = m->ptrn + m 
C                m
C The functions P (x) are normalized such that 
C                n
C                          /   m     2
C                          | [P  (x)] dx = 1/2
C                          /   n
C                             __
C and must be multiplied by  |2  to match Belousov tables.
C                           \|
C                  m
C The derivatives H (x) are defined as 
C                  n        m           2    m
C                          H (x) = -(1-x ) dP (x)/dx
C                           n                n
C
C and are evaluated using the recurrence relationship
C                          _________________________
C      m          m       |  2   2                     m
C     H (x) = nx P (x) -  |(n - m )(2n + 1)/(2n - 1)  P   (x)
C      n          n      \|                            n-1
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      L. Bath, June 1992
C Reviewed:          J. Hack, D. Williamson, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C-----------------------------------------------------------------------
#include "pspect.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer ix                  ! Dimension of Legendre funct arrays
#ifdef WORD4
      real*8 x1                     ! sin of latitude, [sin(phi), or mu]
#else
      real x1                     ! sin of latitude, [sin(phi), or mu]
#endif
C                                 ! and cosine of colatitude
C Output arguments
C
#ifdef WORD4
      real*8 pmn(ix),               ! Legendre function array
     $     hmn(ix)                ! Derivative array
#else
      real pmn(ix),               ! Legendre function array
     $     hmn(ix)                ! Derivative array
#endif
C
C---------------------------Local variables-----------------------------
C
#ifdef WORD4
      real*8 xtemp(3,pmmax+ptrn+1)  ! Workspace for evaluating recurrence
#else
      real xtemp(3,pmmax+ptrn+1)  ! Workspace for evaluating recurrence
#endif
C                                 ! relation where xtemp(m-2,n) and
C                                 ! xtemp(m-1,n) contain Pmn-s required 
C                                 ! to evaluate xtemp(m,n) (i.e.,always
C                                 ! contains three adjacent columns of
C                                 ! the Pmn data structure)
C
      integer jmax,      ! Loop limit (N+1=> 2D wavenumber limit +1)
     $        nmax,      ! Large enough n to envelope truncation
     $        n,         ! 2-D wavenumber index (up/down column)
     $        ml,        ! intermediate scratch variable
     $        k,         ! counter on terms in trig series expansion
     $        n2,        ! 2*n
     $        m,         ! zonal wavenumber index
     $        nto,       ! intermediate scratch variable
     $        mto,       ! intermediate scratch variable
     $        j,         ! 2-D wavenumber index in recurrence evaluation
     $        nmaxm      ! loop limit in recurrence evaluation
C
#ifdef WORD4
      real*8 xte,          ! cosine latitude [cos(phi)]
     $     teta,         ! pi/2 - latitute (colatitude)
     $     an,           ! coefficient on trig. series expansion
     $     sinpar,       ! accumulator in trig. series expansion 
     $     cospar,       ! accumulator in trig. series expansion 
     $     p,            ! 2-D wavenumber (series expansion)
     $     q,            ! intermediate variable in series expansion 
     $     r,            ! zonal wavenumber (recurrence evaluation)
     $     p2,           ! intermediate variable in series expansion 
     $     rr,           ! twice the zonal wavenumber (recurrence)
     $     sqp,          ! intermediate variable in series expansion 
     $     cosfak,       ! coef. on cos term in series expansion
     $     sinfak,       ! coef. on sin term in series expansion
     $     ateta,        ! intermediate variable in series expansion 
     $     costet,       ! cos term in trigonometric series expansion
     $     sintet        ! sin term in trigonometric series expansion
C
      real*8 t,            ! intermediate variable (recurrence evaluation)
     $     wm2,          ! intermediate variable (recurrence evaluation)
     $     wmq2,         ! intermediate variable (recurrence evaluation)
     $     w,            ! intermediate variable (recurrence evaluation)
     $     wq,           ! intermediate variable (recurrence evaluation)
     $     q2,           ! intermediate variable (recurrence evaluation)
     $     wt,           ! intermediate variable (recurrence evaluation)
     $     q2d,          ! intermediate variable (recurrence evaluation)
     $     cmn,          ! cmn  recurrence coefficient (see Belousov)
     $     xdmn,         ! dmn  recurrence coefficient (see Belousov)
     $     emn           ! emn  recurrence coefficient (see Belousov)
#else
      real xte,          ! cosine latitude [cos(phi)]
     $     teta,         ! pi/2 - latitute (colatitude)
     $     an,           ! coefficient on trig. series expansion
     $     sinpar,       ! accumulator in trig. series expansion 
     $     cospar,       ! accumulator in trig. series expansion 
     $     p,            ! 2-D wavenumber (series expansion)
     $     q,            ! intermediate variable in series expansion 
     $     r,            ! zonal wavenumber (recurrence evaluation)
     $     p2,           ! intermediate variable in series expansion 
     $     rr,           ! twice the zonal wavenumber (recurrence)
     $     sqp,          ! intermediate variable in series expansion 
     $     cosfak,       ! coef. on cos term in series expansion
     $     sinfak,       ! coef. on sin term in series expansion
     $     ateta,        ! intermediate variable in series expansion 
     $     costet,       ! cos term in trigonometric series expansion
     $     sintet        ! sin term in trigonometric series expansion
C
      real t,            ! intermediate variable (recurrence evaluation)
     $     wm2,          ! intermediate variable (recurrence evaluation)
     $     wmq2,         ! intermediate variable (recurrence evaluation)
     $     w,            ! intermediate variable (recurrence evaluation)
     $     wq,           ! intermediate variable (recurrence evaluation)
     $     q2,           ! intermediate variable (recurrence evaluation)
     $     wt,           ! intermediate variable (recurrence evaluation)
     $     q2d,          ! intermediate variable (recurrence evaluation)
     $     cmn,          ! cmn  recurrence coefficient (see Belousov)
     $     xdmn,         ! dmn  recurrence coefficient (see Belousov)
     $     emn           ! emn  recurrence coefficient (see Belousov)
#endif
C
C Begin procedure by evaluating the first two columns of the Legendre
C function matrix (i.e., all n for m=0,1) via a trigonometric series 
C expansion (see eqs. 19 and 21 in Belousov, 1962).  Note that indexing
C is offset by one (e.g., m index for wavenumber m=0 is 1 and so on)
C Setup first ...
C
      jmax = ptrn + 1
      nmax = pmmax + jmax
      xte = sqrt(1.-x1*x1)
      teta = acos(x1)
      an = 1.
      xtemp(1,1) = 0.5    ! P00
C
C begin loop over n (2D wavenumber, or degree of associated Legendre 
C function) beginning with n=1 (i.e., P00 was assigned above)
C note n odd/even distinction yielding 2 results per n cycle
C
      do 20 n=2,nmax
         sinpar = 0.
         cospar = 0.
         ml = n
         p = n - 1
         p2 = p*p
         sqp = 1./sqrt(p2+p)
         an = an*sqrt(1.-1./(4*p2))
         cosfak = 1.
         sinfak = p*sqp
         do 14 k=1,ml,2
            q = k - 1
            ateta = (p-q)*teta
            costet = cos(ateta)
            sintet = sin(ateta)
            if (n.eq.k) costet = costet*0.5
            if (k.ne.1) then
               cosfak = (q-1.)/q*(p+p-q+2)/(p+p-q+1.)*cosfak
               sinfak = cosfak*(p-q)*sqp
            end if
            cospar = cospar + costet*cosfak
            sinpar = sinpar + sintet*sinfak
   14    continue
         xtemp(1,n)   = an*cospar       ! P0n vector
         xtemp(2,n-1) = an*sinpar       ! P1n vector
   20 continue
C
C Assign Legendre functions and evaluate derivatives for all n and m=0,1
C
      pmn(1) = 0.5
      pmn(1+jmax) = xtemp(2,1)
      hmn(1) = 0.
      hmn(1+jmax) = x1*xtemp(2,1)
      do 21 n=2,jmax
         pmn(n) = xtemp(1,n)
         pmn(n+jmax) = xtemp(2,n)
         n2 = n + n
         hmn(n) = float(n-1)*(x1*xtemp(1,n)-
     $           sqrt(float(n2-1)/float(n2-3))
     $           *xtemp(1,n-1))
         hmn(n+jmax) = float(n)*x1*xtemp(2,n)-
     $                sqrt(float((n2+1)*(n*n-1))/
     $                float(n2-1))*xtemp(2,n-1)
   21 continue
C
C Evaluate recurrence relationship for remaining Legendre functions
C (i.e., m=2 ... PTRM) and associated derivatives (see eq 17, Belousov)
C
      do 30 m=3,pmmax
         r = m - 1
         rr = r + r
         xtemp(3,1) = sqrt(1.+1./rr)*xte*xtemp(2,1)
         nto = (m-1)*jmax
         pmn(nto+1) = xtemp(3,1)
         hmn(nto+1) = r*x1*xtemp(3,1)
         nmaxm = nmax - m
C
C Loop over 2-D wavenumber (i.e., degree of Legendre function)
C Pmn-s and Hmn-s for current zonal wavenumber, r
C
         do 24 j=2,nmaxm
            mto = nto + j
            t = j - 1
            q = rr + t - 1
            wm2 = q + t
            w = wm2 + 2
            wq = w*q
            q2 = q*q - 1
            wmq2 = wm2*q2
            wt = w*t
            q2d = q2 + q2
            cmn = sqrt((wq*(q-2.))/(wmq2-q2d))
            xdmn = sqrt((wq*(t+1.))/wmq2)
            emn = sqrt(wt/((q+1.)*wm2))
            xtemp(3,j) = cmn*xtemp(1,j) -
     $                 x1*(xdmn*xtemp(1,j+1)-emn*xtemp(3,j-1))
            pmn(mto) = xtemp(3,j)
            hmn(mto) = (r+t)*x1*xtemp(3,j) -
     $                sqrt(wt*(q+1.)/wm2)*xtemp(3,j-1)
   24    continue
C
C shift Pmn-s to left in workspace (setup for next recurrence pass)
C
         do n=1,nmax
            xtemp(1,n) = xtemp(2,n)
            xtemp(2,n) = xtemp(3,n)
         end do
   30 continue
C
      return
      end
