#include "params.h"
      subroutine radalb(lat     ,ioro    ,sndpth  ,coszrs  ,albs    ,
     $                  albl    ,albsd   ,albld   )
C-----------------------------------------------------------------------
C
C Compute surface albedos
C
C Computes surface albedos for direct/diffuse incident radiation for
C two spectral intervals:
C   s = 0.2-0.7 micro-meters
C   l = 0.7-5.0 micro-meters
C
C Uses knowledge of surface type to specify albedo, as follows:
C
C Ocean           Uses solar zenith angle to compute albedo for direct
C                 radiation; diffuse radiation values constant; albedo
C                 independent of spectral interval and other physical
C                 factors such as ocean surface wind speed.
C
C Land without    Albedos specified by two dimensional surface albedo
C     snow        fields, which distinguish surfaces with strong solar
C                 zenith angle dependence from those with weaker solar
C                 zenith angle dependence; alb independent of surface
C                 moisture or other physical factors.
C
C Land with snow  Snow depth (liquid water equivalent) used, along with
C                 aerodynamic roughness to define a horizontal fraction
C                 of land surface covered with snow; snow albedos are
C                 comptd as functions of solar zenith angle; these snow
C                 albedos are then weighted by the horizontal fraction
C                 of coverage with the underlying surface albedos
C                 computed above to produce total grid mean albedo.
C
C land with ice   Surface albedos specified as functions of spectral
C                 interval; combined with overlying snow in a similar
C                 manner to the case of land with snow.
C
C Ocean with      Surface albs specified; combined with overlying snow
C   sea ice       in a similar manner to the case of land with snow.
C
C Note, the code collects together surfaces of the same type for various
C computations in order to vectorize longitude loops.
C
C For more details , see Briegleb, Bruce P., 1992: Delta-Eddington
C Approximation for Solar Radiation in the NCAR Community Climate Model,
C Journal of Geophysical Research, Vol 97, D7, pp7603-7612).
C
C The details of the land surface albedo arrays can be found in the
C common block description below.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, B. Briegleb, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "prgrid.com"
C-----------------------------------------------------------------------
      real snws,           ! Snow albedo for 0.2-0.7 micro-meters
     $     snwl,           ! Snow albedo for 0.7-5.0 micro-meters
     $     sices,          ! Sea ice albedo for 0.2-0.7 micro-meters
     $     sicel           ! Sea ice albedo for 0.7-5.0 micro-meters
      parameter (snws  = 0.95,
     $           snwl  = 0.70,
     $           sices = 0.70,
     $           sicel = 0.50)
C------------------------------Commons----------------------------------
#include "crdalb.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer lat          ! Lat index for two dimensional data arrays
      integer ioro(plond)  ! Surface type flag (ocean, land, sea ice)
      real sndpth(plond),  ! Snow depth (liquid water equivalent)
     $     coszrs(plond)   ! Cosine solar zenith angle
C
C Output arguments
C
      real albs(plond),    ! Srf alb for direct rad   0.2-0.7 micro-ms
     $     albl(plond),    ! Srf alb for direct rad   0.7-5.0 micro-ms
     $     albsd(plond),   ! Srf alb for diffuse rad  0.2-0.7 micro-ms
     $     albld(plond)    ! Srf alb for diffuse rad  0.7-5.0 micro-ms
C
C---------------------------Local variables-----------------------------
C
      integer i,           ! Longitude index
     $        ii,          ! Lngtd indx pnts satisfying certain condtns
     $        ipos(plond), ! Flag for computation points
     $        indx(plond), ! Indices for computation points
     $        npts         ! Number of computation points
C
      logical ilf(plond)   ! Logical for specifying certain types sfc
C
      real ocean(plond),   ! Ocean flag   (=1 for ocean,   0 otherwise)
     $     sice(plond),    ! Sea ice flag (=1 for sea ice, 0 otherwise)
     $     rs,             ! Empirical fact strng znth angl dependence
     $     rw,             ! Empirical fact weak  znth angl dependence
     $     frsnow(plond),  ! Horizontal fraction of snow cover
     $     snwhgt,         ! Physical snow height
     $     rghsnw          ! Roughness for horizontal snow cover fractn
C
      real salbs(plond),   ! Snow alb for direct rad  0.2-0.7 micro-ms
     $     salbl(plond),   ! Snow alb for direct rad  0.7-5.0 micro-ms
     $     salbsd(plond),  ! Snow alb for diffuse rad  0.2-0.7 micro-ms
     $     salbld(plond)   ! Snow alb for diffuse rad  0.7-5.0 micro-ms
C
C------------------------------Externals--------------------------------
C
      external  wheneq     ! When equal funct gives indices for condtn
C
C-----------------------------------------------------------------------
C
C Set flags for ocean (with or without sea ice) and
C ocean with sea ice points:
C
CDIR$ IVDEP
      do i=1,plon
         ocean(i) = 0.
         sice(i)  = 0.
         if(ioro(i).eq.0) ocean(i) = 1.
         if(ioro(i).eq.2) sice(i)  = 1.
      end do
C
C Initialize all surface albedos to zero
C
      do i=1,plon
         albs(i)  = 0.
         albl(i)  = 0.
         albsd(i) = 0.
         albld(i) = 0.
      end do
C
C Land surfaces
C
      do i=1,plon
         if(ocean(i).ne.1.0 .and. sice(i).eq.0.0 .and.
     $      coszrs(i).gt.0.0) then
              ipos(i) = 1
         else
              ipos(i) = 0
         end if
      end do
      call wheneq(plon,ipos,1,1,indx,npts)
CDIR$ IVDEP
      do ii=1,npts
         i = indx(ii)
C
C Use empirical factors to adjust surface albedos for zenith angle
C effects, distinquishing between strong and weakly dependent surfaces:
C
         rs = 1.4/(1. + .8*coszrs(i))
         rw = 1.1/(1. + .2*coszrs(i))
         albs(i)  = albvss(i,lat)*frctst(i,lat)*rs +
     $              albvsw(i,lat)*(1. - frctst(i,lat))*rw
         albl(i)  = albnis(i,lat)*frctst(i,lat)*rs +
     $              albniw(i,lat)*(1. - frctst(i,lat))*rw
         albsd(i) = albvss(i,lat)*frctst(i,lat) +
     $              albvsw(i,lat)*(1. - frctst(i,lat))
         albld(i) = albnis(i,lat)*frctst(i,lat) +
     $              albniw(i,lat)*(1. - frctst(i,lat))
      end do
C
C Surfaces covered with sea ice:
C
      do i=1,plon
         if( (sice(i).eq.1.0 .and. coszrs(i).gt.0.0) ) then
           ipos(i) = 1
         else
           ipos(i) = 0
         end if
      end do
C
      call wheneq(plon,ipos,1,1,indx,npts)
CDIR$ IVDEP
      do ii=1,npts
         i = indx (ii)
         albs(i)  = sices
         albl(i)  = sicel
         albsd(i) = albs(i)
         albld(i) = albl(i)
      end do
C
C Points with snowcover:
C
      do i=1,plon
         ilf(i) = sndpth(i) .gt. 0. .and. coszrs(i) .gt. 0.
         if( sndpth(i).gt.0. .and. coszrs(i).gt.0. ) then
           ipos(i) = 1
         else
           ipos(i) = 0
         end if
      end do
C
      call wheneq(plon,ipos,1,1,indx,npts)
      do ii=1,npts
         i = indx(ii)
         salbsd(i) = snws
         salbld(i) = snwl
      end do
C
C Direct snow albedos: distinguish between 2 zenith angle regimes
C
      do i=1,plon
         ipos(i) = 0
         if(ilf(i)) ipos(i) = 2
         if(ilf(i) .and. coszrs(i).lt.0.5) ipos(i) = 1
      end do
C
C Zenith angle regime 1 ( coszrs < 0.5 ).
C Set direct snow albedos (limit to 0.98 max)
C
      call wheneq(plon,ipos,1,1,indx,npts)
      do ii=1,npts
         i        = indx(ii)
         salbs(i) = amin1(0.98,salbsd(i) + (1. - salbsd(i))*0.5*
     $                    ((3./(1. + 4.*coszrs(i))) - 1.))
         salbl(i) = amin1(0.98,salbld(i) + (1. - salbld(i))*0.5*
     $                    ((3./(1. + 4.*coszrs(i))) - 1.))
      end do
C
C Zenith angle regime 2 ( coszrs >= 0.5 )
C
      call wheneq(plon,ipos,1,2,indx,npts)
      do ii=1,npts
         i        = indx(ii)
         salbs(i) = snws
         salbl(i) = snwl
      end do
C
C Compute fraction of horizonal surface covered with snow:
C
      do i=1,plon
         if(sndpth(i) .le. 0.) then
            snwhgt    = 0.0
            rghsnw    = 0.0
            frsnow(i) = 0.0
         else
            snwhgt    = 20. * sndpth(i)
            rghsnw    = amax1(rghnss(i,lat),0.25)
            frsnow(i) = snwhgt/(rghsnw + snwhgt)
         end if
      end do
C
C Points with snowcover:
C
      do i=1,plon
         if( sndpth(i).gt.0. .and. coszrs(i).gt.0. ) then
           ipos(i) = 1
         else
           ipos(i) = 0
         end if
      end do
      call wheneq(plon,ipos,1,1,indx,npts)
C
C Compute both diffuse and direct total albedos
C
CDIR$ IVDEP
      do ii=1,npts
         i = indx(ii)
         albs(i)  = albs(i) *(1.-frsnow(i)) + salbs(i) *frsnow(i)
         albl(i)  = albl(i) *(1.-frsnow(i)) + salbl(i) *frsnow(i)
         albsd(i) = albsd(i)*(1.-frsnow(i)) + salbsd(i)*frsnow(i)
         albld(i) = albld(i)*(1.-frsnow(i)) + salbld(i)*frsnow(i)
      end do
C
C Local points over ice-free ocean with incident solar radiation:
C
      do i=1,plon
         ipos(i)=0
         if(ocean(i).eq.1. .and. sice(i).eq.0. .and. coszrs(i).ge.0.)
     $     ipos(i) = 1
      end do
      call wheneq(plon,ipos,1,1,indx,npts)
C
C Ocean albedos function of solar zenith angle only,and
C independent of spectral interval:
C
CDIR$ IVDEP
      do ii=1,npts
         i = indx(ii)
         albl(i)  = (.026/(coszrs(i)**1.7 + .065)) +
     $              (.15*(coszrs(i) - 0.10)*
     $                   (coszrs(i) - 0.50)*
     $                   (coszrs(i) - 1.00)  )
         albs(i)  = albl(i)
         albld(i) = 0.06
         albsd(i) = 0.06
      end do
C   
      return
      end
