#include "params.h"
      subroutine radctl(hbuf    ,lhbuf   ,
     $                  lat     ,clat    ,ioro    ,sndpth  ,
     $                  ts      ,pmid    ,pint    ,pmln    ,piln    ,
     $                  t       ,h2ommr  ,cld     ,effcld  ,clwp    ,
     $                  plol    ,plos    ,fsns    ,qrs     ,qrl     ,
     $                  flwds   ,coslat  ,rel     ,rei     ,fice)
C-----------------------------------------------------------------------
C
C Driver for radiation computation.
C
C Radiation uses cgs units, so conversions must be done from
C model fields to radiation fields.
C
C Calling sequence:
C
C     radoz2      Interpolates ozone paths to model interface pressures
C
C     radinp      Converts units of model fields and computes ozone
C                 mixing ratio for solar scheme
C
C     radcsw      Performs solar computation
C       radalb    Computes surface albedos
C       radded    Computes delta-Eddington solution
C       radclr    Computes diagnostic clear sky fluxes
C
C     radclw      Performs longwave computation
C
C       radtpl    Computes path quantities
C       radems    Computes emissivity
C       radabs    Computes absorptivity
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, B. Briegleb, August 1992
C
C  CCM3 physics and FOAM additions:  R. Jacob, December 1996
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "ptrrgrid.com"
#include "pagrid.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------

#ifdef COUPLED
c albedos come from the coupler via the land surface model
#include "pccmcoup.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real hbuf(*)              ! history tape buffer,for calls to outfld
      integer lat               ! Latitude row index
      integer lhbuf             ! history tape buffer length
      real clat                 ! Current latitude (radians)
      integer ioro(plond)       ! Land/ocean/sea ice flag
      real sndpth(plond),       ! Snow depth (liquid water equivalent)
     $     ts(plond),           ! Surface (skin) temperature
     $     pmid(plond,plev),    ! Model level pressures
     $     pint(plond,plevp),   ! Model interface pressures
     $     pmln(plond,plev),    ! Natural log of pmid
     $     piln(plond,plevp),   ! Natural log of pint
     $     t(plond,plev),       ! Model level temperatures
     $     h2ommr(plond,plev),  ! Model level specific humidity
     $     cld(plond,plevp),    ! Fractional cloud cover
     $     effcld(plond,plevp), ! Effective fractional cloud cover
     $     clwp(plond,plev),    ! Cloud liquid water path
     $     plol(plond,plevp),   ! O3 pressure weighted path lengths (cm)
     $     plos(plond,plevp)    ! O3 path lengths (cm)

c  CCM3 additions
      real coslat               ! cosine latitude
      real rel(plond,plev)      ! liquid effective drop size (microns)
      real rei(plond,plev)      ! ice effective drop size (microns)
      real fice(plond,plev)     ! fractional ice content within cloud

C
C Output solar arguments
C
      real fsns(plond),         ! Surface absorbed solar flux
     $     qrs(plond,plev)      ! Solar heating rate
C
C Output longwave arguments
C
      real qrl(plond,plev),     ! Longwave cooling rate
     $     flwds(plond)         ! Surface down longwave flux
C
C---------------------------Local variables-----------------------------
C
      integer i                 ! index
      real solin(plond),        ! Solar incident flux
     $     fsnt(plond),         ! Net column abs solar flux at model top
     $     fsntc(plond),        ! Clear sky total column abs solar flux
     $     fsnsc(plond)         ! Clear sky surface abs solar flux
      real flnt(plond),         ! Net outgoing lw flux at model top
     $     flns(plond),         ! Srf longwave cooling (up-down) flux
     $     flntc(plond),        ! Clear sky lw flux at model top
     $     flnsc(plond)         ! Clear sky lw flux at srf (up-down)
      real pbr(plond,plevr),    ! Model mid-level pressures (dynes/cm2)
     $     pnm(plond,plevrp),   ! Model interface pressures (dynes/cm2)
     $     o3mmr(plond,plevr),  ! Ozone mass mixing ratio
     $     plco2(plond,plevrp), ! Prs weighted CO2 path
     $     plh2o(plond,plevrp), ! Prs weighted H2O path
     $     tclrsf(plond,plevrp),! Total clear sky fraction, level to space
     $     coszrs(plond),       ! Cosine solar zenith angle
     $     eccf                 ! Earth/sun distance factor
      real tmp(plond)           ! Temporary for outfld
c albedos come from the coupler via the land surface model
      real albs(plond),    ! Srf alb for direct rad   0.2-0.7 micro-ms
     $     albl(plond),    ! Srf alb for direct rad   0.7-5.0 micro-ms
     $     albsd(plond),   ! Srf alb for diffuse rad  0.2-0.7 micro-ms
     $     albld(plond)    ! Srf alb for diffuse rad  0.7-5.0 micro-ms

c CCM3 local arrays
      real o3vmr(plond,plevr)   ! Ozone volume mixing ratio
      real aermmr(plond,plevr)  ! level aerosol mass mixing ratio
      real rh(plond,plevr)      ! level relative humidity (fraction)
      real n2o(plond,plev)      ! nitrous oxide mass mixing ratio
      real ch4(plond,plev)      ! methane mass mixing ratio
      real cfc11(plond,plev)    ! cfc11 mass mixing ratio
      real cfc12(plond,plev)    ! cfc12 mass mixing ratio
      save o3vmr



C
C Declare local arrays to which model input arrays are interpolated here.
C Current default is none since radiation grid = model grid.
C
C Externals.
C
      external radinp,          ! Computes latitude dependent radiation input
     $         radcsw,          ! Computes solar radiation
     $         radclw,          ! Computes longwave radiation
     $         radxfr,          ! Exchange data point to balance the load
     $         torgrid,         ! Interpolate model variables to radiation grid
     $         fmrgrid          ! Interpolate radiation variables to model grid
C--------------------------------------------------------------------------
C
C Interpolate model input arrays to radiation vertical grid.  Currently this 
C is a do-nothing routine because radiation grid = model grid.
C
      call torgrid(pmid    ,pint    ,pmln    ,piln    ,t       ,
     $             h2ommr  ,cld     ,effcld  ,clwp    ,
     $             pmid    ,pint    ,pmln    ,piln    ,t       ,
     $             h2ommr  ,cld     ,effcld  ,clwp    )
C
C Set ozone
C
      call radozn(lat     ,pmid    ,o3vmr   )
C
C Set latitude dependent radiation input
C
C add o3vmr for CCM3 version
      call radinp(dodiavg ,clat    ,pmid    ,pint    ,h2ommr  ,
     $            cld     ,plos    ,pbr     ,pnm     ,plco2   ,
     $            plh2o   ,tclrsf  ,eccf    ,coszrs  ,o3mmr   ,
     $		  o3vmr)
C
C Solar radiation computation
C
      if (dosw) then

C call the new CCM3 aerosol mass mixing ratio routine
        call aermix(pnm     ,aermmr  ,rh      )

#ifdef COUPLED
      do i=1,plon
	 albs(i) = from_coupler(i,lat,15)
	 albl(i) = from_coupler(i,lat,16)
	 albsd(i) = from_coupler(i,lat,17)
	 albld(i) = from_coupler(i,lat,18)
      enddo
#else
      call radalb(lat     ,ioro    ,sndpth  ,coszrs  ,albs    ,
     $            albl    ,albsd   ,albld   )
#endif


c load balance.  Swap half the lighted longs with half the dark longs
c
#if defined(PP_RADLB) && (PP_NPROC_LN != 1)
         call radxfr(lat   ,ioro   ,sndpth ,pnm    ,h2ommr  ,
     $               cld   ,clwp   ,o3mmr  ,coszrs ,solin   ,
     $               qrs   ,fsns   ,fsnt   ,fsnsc  ,fsntc   ,
     $		     t     ,aermmr ,rh     ,rel    ,rei     ,
     $               fice  ,
     $		     albs, albl, albsd, albld,
     $               .true.)
#endif
C
C CCM3 version
	call radcsw(pnm     ,h2ommr  ,o3mmr   ,aermmr  ,rh      ,
     $              cld     ,clwp    ,rel     ,rei     ,fice    ,
     $              eccf    ,coszrs  ,
     $              albs, albsd, albl, albld,
     $                       solin   ,qrs     ,fsns    ,fsnt    ,
     $              fsnsc   ,fsntc )

C
#if defined(PP_RADLB) && (PP_NPROC_LN != 1)
         call radxfr(lat   ,ioro   ,sndpth ,pnm    ,h2ommr  ,
     $               cld   ,clwp   ,o3mmr  ,coszrs ,solin   ,
     $               qrs   ,fsns   ,fsnt   ,fsnsc  ,fsntc   ,
     $		     t     ,aermmr ,rh     ,rel    ,rei     ,
     $               fice  ,
     $		     albs, albl, albsd, albld,
     $               .false.)
#endif
C
C Convert units of shortwave fields needed by rest of model from CGS to MKS
C
         do i=1,plon
            solin(i) = solin(i)*1.e-3
            fsnt(i)  = fsnt(i) *1.e-3
            fsns(i)  = fsns(i) *1.e-3
            fsntc(i) = fsntc(i)*1.e-3
            fsnsc(i) = fsnsc(i)*1.e-3
         end do
C
C Calculate/outfld albedo and clear sky albedo
C
         if (ninavg(1).eq.'Q') then
            do i=1,plon
               if (solin(i).gt.0.) then
                  tmp(i) = (solin(i) - fsnt(i)) / solin(i)
               else
                  tmp(i) = 0.
               end if
            end do
            call outfld('ALB     ',tmp ,plond,lat,hbuf,lhbuf)
C
            do i=1,plon
               if (solin(i).gt.0.) then
                  tmp(i) = (solin(i) - fsntc(i)) / solin(i)
               else
                  tmp(i) = 0.
               end if
            end do
            call outfld('ALBCLR  ',tmp ,plond,lat,hbuf,lhbuf)
         end if
C
C Dump shortwave radiation information to history tape buffer (diagnostics)
C
         call outfld('SOLIN   ',solin ,plond,lat,hbuf,lhbuf)
         call outfld('FSNT    ',fsnt  ,plond,lat,hbuf,lhbuf)
         call outfld('FSNS    ',fsns  ,plond,lat,hbuf,lhbuf)
         call outfld('FSNTC   ',fsntc ,plond,lat,hbuf,lhbuf)
         call outfld('FSNSC   ',fsnsc ,plond,lat,hbuf,lhbuf)
      end if
C
C Longwave radiation computation
C
      if (dolw) then

c
c Specify trace gas mixing ratios - a CCM3 addition
c
         call trcmix(pmid, clat, coslat, n2o, ch4, cfc11, cfc12)

C call the CCM3 version
	call radclw(lat     ,ts      ,t       ,h2ommr  ,o3vmr   ,
     $              pbr     ,pnm     ,pmln    ,piln    ,plco2   ,
     $              plh2o   ,n2o     ,ch4     ,cfc11   ,cfc12   ,
     $              effcld  ,tclrsf  ,qrl     ,flns    ,flnt    ,
     $              flnsc   ,flntc   ,flwds   )

c        call radclw(lat     ,ts      ,plol    ,plos    ,t       ,
c    $               h2ommr  ,pbr     ,pnm     ,pmln    ,piln    ,
c    $               plco2   ,plh2o   ,effcld  ,tclrsf  ,qrl     ,
c    $               flns    ,flnt    ,flnsc   ,flntc   ,flwds   )
C
C Convert units of longwave fields needed by rest of model from CGS to MKS
C
         do i=1,plon
            flnt(i)  = flnt(i)*1.e-3
            flns(i)  = flns(i)*1.e-3
            flntc(i) = flntc(i)*1.e-3
            flnsc(i) = flnsc(i)*1.e-3
            flwds(i) = flwds(i)*1.e-3
         end do
C
C Dump longwave radiation information to history tape buffer (diagnostics)
C
         call outfld('FLNT    ',flnt  ,plond,lat,hbuf,lhbuf)
         call outfld('FLNTC   ',flntc ,plond,lat,hbuf,lhbuf)
         call outfld('FLNS    ',flns  ,plond,lat,hbuf,lhbuf)
         call outfld('FLNSC   ',flnsc ,plond,lat,hbuf,lhbuf)
      end if
C
C Interpolate radiation output arrays to model vertical grid.  Currently this 
C is a do-nothing routine because radiation grid = model grid.
C
      call fmrgrid(qrs     ,qrl     ,
     $             qrs     ,qrl     )
C
      return
      end
