#include "params.h"
      subroutine cmpsnw(lat     ,clat    ,ioro    ,snwcvr  )
C-----------------------------------------------------------------------
C
C Compute snow depth
C
C Compute geographic distribution of seasonally varying snow cover
C (liquid water equivalent) based on january and july climatological
C snow covers
C
C In the following, 'nh' refers to northern hemisphere, and 
C 'sh' to southern hemisphere.
C
C 'sdphse' defines the calendar day of year for which the nh january
C          snow cover (assumed to be the maximum value) is used  
C 'slphse' defines calendar day when nh snow line furthest equatorward
C 'dphinh' defines meridional range of nh snow line (in degrees)
C 'dphish' defines meridional range of sh snow line (in degrees)
C 'p0max'  defines poleward extent of snow line in both hemispheres
C 'sltrns' defines meridional extent of snow line transition region
C 'snwmin' is minimum snow cover below which value is set to zero
C 'rlat'   is the current latitude (units of degrees)
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Hack, B. Briegleb
C Standardized:      J. Rosinski, June 1992
C Reviewed:          B. Briegleb, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C-----------------------------------------------------------------------
      real slphse,       ! cal day when nh snow line furthest equatorward
     $     sdphse,       ! calendar day for nh snow cover maximum
     $     dphinh,       ! meridional range snow line in northern hem
     $     dphish,       ! meridional range snow line in southern hem
     $     p0max ,       ! poleward extent of snow in both hemispheres
     $     sltrns,       ! meridional extent of snow line trans. region
     $     snwmin        ! min snow depth below which value set to zero
C
      parameter (slphse = 46.0,
     $           sdphse = 31.0,
     $           dphinh = 60.0,
     $           dphish = 40.0,
     $           p0max  = 90.0,
     $           sltrns = 10.0,
     $           snwmin = .005)
C------------------------------Commons----------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "crdalb.com"
C-----------------------------------------------------------------------
#include "crdcon.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     lat           ! latitude index
      real clat          ! latitude in radians
      integer
     $     ioro(plond)   ! land/ocean/sea ice flag
C
C Output arguments
C
      real snwcvr(plond) ! snow depth field
C
C---------------------------Local workspace-----------------------------
C
      integer
     $     i             ! longitude index
      real rlat,         ! current latitude in degrees
     $     tmefac,       ! general time factor
     $     sdnh  ,       ! northern hemisphere time factor
     $     sdsh  ,       ! souther  heisphere time factor
     $     p0nh  ,       ! adjustable north hem snow line parameter
     $     p0sh  ,       ! adjustable southern hem snow line param
     $     snwmn ,       ! min snow cover for hemisphere
     $     amp   ,       ! jan - july (july - jan) snow cover diff
     $     fs            ! factor for snow line transition region
C
C-----------------------------------------------------------------------
C
      rlat   = (180./pie)*clat
C
C Compute snow depth parameters for both hemispheres
C by shifting cos function calculated by tmefac up or down
C
      tmefac = 0.50*cos(2.0*pie*(calday - sdphse)/dayspy)
      sdnh   = 0.5 + tmefac
      sdsh   = 0.5 - tmefac
C
C Compute snow line parameters for both hemispheres
C similar to snow depth procedure
C
      tmefac = 0.50*cos(2.0*pie*(calday - slphse)/dayspy)
      p0nh   =  p0max - dphinh*(0.5 + tmefac)
      p0sh   = -p0max + dphish*(0.5 - tmefac)
C
C Decide which hemisphere we re in and take the appropriate action
C fs is a parameter related to location of snow line. the
C product (fs*sdxh*amp)+minsnw determines actual snow cover;
C i.e., fs modulates the snow depth sdxh*amp where x equals n or s
C
      if (rlat.gt.0.0) then
C
C Northern hemisphere
C
         if (rlat.ge.p0nh) fs = 1.0
         if (rlat.lt.p0nh .and. rlat.gt.(p0nh - sltrns)) then
            fs = 0.50*(1.0 + cos(pie*(rlat - p0nh)/sltrns))
         end if
         if (rlat.le.(p0nh - sltrns)) fs = 0.0
         do i=1,plon
            amp   = snwjan(i,lat) - snwjly(i,lat)
            snwmn = snwjly(i,lat)
            if ( ioro(i) .eq. 1 ) then
               snwcvr(i) = fs*sdnh*amp + snwmn
            end if
            if (snwcvr(i).lt.snwmin) snwcvr(i) = 0.0
         end do
      else
C
C Southern hemisphere
C
         if (rlat.le.p0sh) fs = 1.0
         if (rlat.gt.p0sh .and. rlat.lt.(p0sh + sltrns)) then
            fs = 0.50*(1.0 + cos(pie*(rlat - p0sh)/sltrns))
         end if
         if (rlat.ge.(p0sh + sltrns)) fs = 0.0
         do i=1,plon
            amp   = snwjly(i,lat) - snwjan(i,lat)
            snwmn = snwjan(i,lat)
            if ( ioro(i) .eq. 1 ) then
               snwcvr(i) = fs*sdsh*amp + snwmn
            end if
            if (snwcvr(i).lt.snwmin) snwcvr(i) = 0.0
         end do
      end if
C
      return
      end
