C#######################################################################
C PSTSWM Version 4.0 (12/1/94)                                         #
C  A message-passing benchmark code and parallel algorithm testbed     #
C  that solves the nonlinear shallow water equations using the spectral#
C  transform method.                                                   #
C Written by:                                                          #
C  Patrick Worley of Oak Ridge National Laboratory                     #
C  Ian Foster of Argonne National Laboratory                           #
C Based on the sequential code STSWM 2.0 by James Hack and Ruediger    #
C  Jakob of the National Center for Atmospheric Research.              #
C Research and development funded by the Computer Hardware, Advanced   #
C  Mathematics, and Model Physics (CHAMMP) program of the U.S.         #
C  Department of Energy.                                               # 
C                                                                      #
C Questions and comments should be directed to worley@msr.epm.ornl.gov #
C Please notify and acknowledge the authors in any research or         #
C publications utilizing PSTSWM or any part of the code.               #
C                                                                      #
C NOTICE: Neither the institutions nor the authors make any            #
C representations about the suitability of this software for any       #
C purpose. This software is provided "as is", without express or       #
C implied warranty.                                                    #
C#######################################################################
C include precision declaration definitions                            #
#include "params.h"
C#######################################################################
      SUBROUTINE RINGMAX(COMMOPT, PROTOPT, RINGSIZE,
     &                   ME, LEFT, RIGHT, MYINDEX, BASE1, BASE2,
     &                   LTH, WS, SUP)
C
C This subroutine calls routines that calculate a vector max over a 
C specified subset of processors using a ring-shift algorithm.
C The results are duplicated across all processors in the subset.
C
C Communication options (COMMOPT) for RINGMAX include:
C  IF (COMMOPT .EQ. 0) simple shift: send/recv/max
C  IF (COMMOPT .EQ. 1) ordered shift: [send/recv]|[recv/send]/max
C  IF (COMMOPT .EQ. 2) simple shift with recv-ahead
C  IF (COMMOPT .EQ. 3) ordered shift with recv-ahead
C Communication protocol options (PROTOPT) for RINGMAX include:
C  IF (PROTOPT .EQ. 1, 3, .OR. 5)     nonblocking send   
C  IF (PROTOPT .EQ. 2, 3, 4, .OR. 5)  nonblocking receive
C  IF (PROTOPT .EQ. 4 .OR. 5)         forcetype          
C  IF (PROTOPT .EQ. 6 .AND. COMMOPT .EQ. 1) synchronous  
C
C called by: 
C calls: RMAX1, RMAX2
C
C---- Implicit None ----------------------------------------------------
C
      IMPLICIT NONE
C
C---- Arguments --------------------------------------------------------
C
C     Input
C
C communication algorithm option
      INTEGER COMMOPT
C communication protocol option 
      INTEGER PROTOPT
C number of processors in subset
      INTEGER RINGSIZE
C my processor id
      INTEGER ME
C processor ids for neighbors in logical ring
      INTEGER LEFT, RIGHT
C index of "me" in logical ring
      INTEGER MYINDEX
C message type offsets to use in interprocessor communication
      INTEGER BASE1, BASE2
C length of vectors 
      INTEGER LTH
C
C     Work Space
C
C message buffers
      REAL WS(LTH)
C
C     Input/Output
C
C on entry, contains (local) data, on exit contains vector max
      REAL SUP(LTH)
C
C---- Executable Statements --------------------------------------------
C
      IF (RINGSIZE .GT. 1) THEN
C       Calculate vector max.
C
        IF (COMMOPT .LE. 1) THEN
C         no recv-ahead algorithms
          CALL RMAX1(COMMOPT, PROTOPT, RINGSIZE, ME, LEFT,
     &               RIGHT, MYINDEX, BASE1, BASE2, LTH, WS, SUP)
C
        ELSEIF (COMMOPT .LE. 3) THEN
C         recv-ahead algorithms
          CALL RMAX2(COMMOPT-2, PROTOPT, RINGSIZE, ME, LEFT,
     &               RIGHT, MYINDEX, BASE1, BASE2, LTH, WS, SUP)
C
        ELSE
C         illegal communication option specified
          WRITE(0,100) ME, COMMOPT
  100     FORMAT (/,' PSTSWM: FATAL ERROR IN SUBROUTINE RINGMAX ',/,
     &            ' ILLEGAL COMMUNICATION OPTION SPECIFIED',/,
     &            ' PROCID = ',I4,' COMMOPT = ',I4)
          STOP
C
        ENDIF
C
      ENDIF
C
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE RMAX1(COMMOPT, PROTOPT, RINGSIZE,
     &                 ME, LEFT, RIGHT, MYINDEX, BASE1, BASE2,
     &                 LTH, WS, SUP)
C
C This subroutine calculates a vector max over a specified subset
C of processors using a ring-shift algorithm without recv-ahead.
C The results are duplicated across all processors in the subset.
C
C Communication options (COMMOPT) for RINGMAX include:
C  IF (COMMOPT .EQ. 0) simple shift: send/recv/max
C  IF (COMMOPT .EQ. 1) ordered shift: [send/recv]|[recv/send]/max
C Communication protocol options (PROTOPT) for RINGMAX include:
C  IF (PROTOPT .EQ. 1, 3, .OR. 5)     nonblocking send   
C  IF (PROTOPT .EQ. 2, 3, 4, .OR. 5)  nonblocking receive
C  IF (PROTOPT .EQ. 4 .OR. 5)         forcetype          
C  IF (PROTOPT .EQ. 6 .AND. COMMOPT .EQ. 1) synchronous  
C
C called by: RINGMAX
C calls: RINGINIT, SENDRECV
C
C---- Implicit None ----------------------------------------------------
C
      IMPLICIT NONE
C
C---- Parameters -------------------------------------------------------
C
#     include "palgor.com"
C
C---- Arguments --------------------------------------------------------
C
C     Input
C
C communication algorithm option
      INTEGER COMMOPT
C communication protocol option 
      INTEGER PROTOPT
C number of processors in subset
      INTEGER RINGSIZE
C my processor id
      INTEGER ME
C processor ids for neighbors in logical ring
      INTEGER LEFT, RIGHT
C index of "me" in logical ring
      INTEGER MYINDEX
C message type offsets to use in interprocessor communication
      INTEGER BASE1, BASE2
C length of vectors
      INTEGER LTH
C
C     Work Space
C
C message buffers
      REAL WS(LTH)
C
C     Input/Output
C
C On entry, SUP contains (local) data. On exit, SUP contains the vector
C max.  
      REAL SUP(LTH)
C
C---- Local Variables --------------------------------------------------
C
C order of send/recv in ordered sendrecv algorithm
      INTEGER ORDER
C arrays indicating the size of the message being sent/received 
C during a given sendrecv, and the corresponding index offset
      INTEGER SIZE(NPROCSX)
      INTEGER OFFSET(NPROCSX)
C work space used to calculate size and index
      INTEGER TEMP(NPROCSX)
C temporaries for current message sizes and offsets
      INTEGER SNDLTH, RCVLTH, SNDDEX, RCVDEX
C loop indices
      INTEGER I, STEP
C
C---- Executable Statements --------------------------------------------
C
C     Precalculate message order, sizes, and offsets.
      CALL RINGINIT(LTH, ALIGN, RINGSIZE, MYINDEX, TEMP, ORDER, SIZE, 
     &              OFFSET)
C
C     Calculate vector max (leaving result distributed).
      DO STEP=RINGSIZE-1,1,-1
C
        SNDLTH = SIZE(STEP+1)
        SNDDEX = OFFSET(STEP+1)
        RCVLTH = SIZE(STEP)
        RCVDEX = OFFSET(STEP)
C
C       Shift vector segment.
        CALL SENDRECV(COMMOPT, PROTOPT, ORDER, ME, 
     &                STEP+BASE1, RIGHT, RBYTES*SNDLTH, SUP(SNDDEX), 
     &                LEFT, RBYTES*RCVLTH, WS(RCVDEX))
C
C       Update vector segment.
        DO I = RCVDEX, RCVDEX+(RCVLTH-1)
          SUP(I) = MAX(SUP(I),WS(I))
        ENDDO
C
      ENDDO
C
C     Broadcast distributed vector by reversing ring-shift algorithm. 
      DO STEP=1,RINGSIZE-1
C
        RCVLTH = SIZE(STEP+1)
        RCVDEX = OFFSET(STEP+1)
        SNDLTH = SIZE(STEP)
        SNDDEX = OFFSET(STEP)
C
C       Shift vector segment.
        CALL SENDRECV(COMMOPT, PROTOPT, ORDER, ME, 
     &                STEP+BASE2, LEFT, RBYTES*SNDLTH, SUP(SNDDEX), 
     &                RIGHT, RBYTES*RCVLTH, SUP(RCVDEX))
C
      ENDDO
C
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE RMAX2(COMMOPT, PROTOPT, RINGSIZE,
     &                 ME, LEFT, RIGHT, MYINDEX, BASE1, BASE2,
     &                 LTH, WS, SUP)
C
C This subroutine calculates a vector max over a specified subset
C of processors using a ring-shift algorithm with recv-ahead.
C The results are duplicated across all processors in the subset.
C
C Communication options (COMMOPT) for RINGMAX include:
C  IF (COMMOPT .EQ. 0) simple shift: send/recv/max
C  IF (COMMOPT .EQ. 1) ordered shift: [send/recv]|[recv/send]/max 
C Communication protocol options (PROTOPT) for RINGMAX include:
C  IF (PROTOPT .EQ. 1, 3, .OR. 5)     nonblocking send   
C  IF (PROTOPT .EQ. 2, 3, 4, .OR. 5)  
C    nonblocking receive and recv-ahead
C  IF (PROTOPT .EQ. 4 .OR. 5)         forcetype          
C  IF (PROTOPT .EQ. 6 .AND. COMMOPT .EQ. 1) synchronous  
C
C called by: RINGMAX
C calls: RINGINIT, SR1, SR2, SR3
C
C---- Implicit None ----------------------------------------------------
C
      IMPLICIT NONE
C
C---- Parameters -------------------------------------------------------
C
#     include "palgor.com"
C
C---- Arguments --------------------------------------------------------
C
C     Input
C
C communication algorithm option
      INTEGER COMMOPT
C communication protocol option 
      INTEGER PROTOPT
C number of processors in subset
      INTEGER RINGSIZE
C my processor id
      INTEGER ME
C processor ids for neighbors in logical ring
      INTEGER LEFT, RIGHT
C index of "me" in logical ring
      INTEGER MYINDEX
C message type offsets to use in interprocessor communication
      INTEGER BASE1, BASE2
C length of vectors
      INTEGER LTH
C
C     Work Space
C
C message buffers
      REAL WS(LTH)
C
C     Input/output
C
C On entry, SUP contains (local) data. On exit, SUP contains the vector
C max. 
      REAL SUP(LTH)
C
C---- Local Variables --------------------------------------------------
C
C order of send/recv in ordered sendrecv algorithm
      INTEGER ORDER
C arrays indicating the size of the message being sent/received 
C during a given sendrecv, and the corresponding index offset
      INTEGER SIZE(NPROCSX)
      INTEGER OFFSET(NPROCSX)
C work space used to calculate size and index
      INTEGER TEMP(NPROCSX)
C temporaries for current message sizes and offsets
      INTEGER SNDLTH, RCVLTH, SNDDEX, RCVDEX
C loop indices
      INTEGER I, STEP
C
C---- Executable Statements --------------------------------------------
C
C     Precalculate message order, sizes, and offsets.
      CALL RINGINIT(LTH, ALIGN, RINGSIZE, MYINDEX, TEMP, ORDER, SIZE, 
     &              OFFSET)
C
C     Post all receive requests for vector max algorithm
C     (disabling safe forcetype protocol - only needs to be used
C     once).
      DO I=RINGSIZE-1,1,-1
        RCVLTH = SIZE(I)
        RCVDEX = OFFSET(I)
        CALL SR1(PROTOPT, .FALSE., ME, I+BASE1, LEFT, 
     &           RBYTES*RCVLTH, WS(RCVDEX))
      ENDDO
C
C     Notify source processor that first set of receive requests
C     have been posted. This trick preserves the order of 
C     SENDRECV requests, which can be more efficient on some machines.
      CALL SRSAFE(PROTOPT, ME, RINGSIZE-1+BASE1, LEFT)
C
C     Post receive requests for broadcast algorithm
C     (since know that requests will be posted before messages
C     sent, can disable safe forcetype protocol).
      DO I=1,RINGSIZE-1
        RCVLTH = SIZE(I+1)
        RCVDEX = OFFSET(I+1)
        CALL SR1(PROTOPT, .FALSE., ME, I+BASE2, RIGHT, 
     &           RBYTES*RCVLTH, SUP(RCVDEX))
      ENDDO
C
C     Calculate vector max using ring-shift algorithm (leaving result 
C     distributed):
C     1) (STEP .EQ. RINGSIZE-1) case: use safe forcetype protocol.
      SNDLTH = SIZE(RINGSIZE)
      SNDDEX = OFFSET(RINGSIZE)
      RCVLTH = SIZE(RINGSIZE-1)
      RCVDEX = OFFSET(RINGSIZE-1)
C
      CALL SR2(COMMOPT, PROTOPT, .TRUE., ORDER, ME, 
     &         RINGSIZE-1+BASE1, RIGHT, RBYTES*SNDLTH, SUP(SNDDEX), 
     &         LEFT, RBYTES*RCVLTH, WS(RCVDEX))
C
C     update vector segment
      DO I = RCVDEX, RCVDEX+(RCVLTH-1)
        SUP(I) = MAX(SUP(I),WS(I))
      ENDDO
C
C     2) (STEP .LT. RINGSIZE-1) case: no safe forcetype protocol.
      DO STEP=RINGSIZE-2,1,-1
C
        SNDLTH = SIZE(STEP+1)
        SNDDEX = OFFSET(STEP+1)
        RCVLTH = SIZE(STEP)
        RCVDEX = OFFSET(STEP)
C
        CALL SR2(COMMOPT, PROTOPT, .FALSE., ORDER, ME, 
     &           STEP+BASE1, RIGHT, RBYTES*SNDLTH, SUP(SNDDEX),
     &           LEFT, RBYTES*RCVLTH, WS(RCVDEX))
C
C       Update vector segment.
        DO I = RCVDEX, RCVDEX+(RCVLTH-1)
          SUP(I) = MAX(SUP(I),WS(I))
        ENDDO
C
      ENDDO
C
C     Broadcast distributed vector by reversing ring-shift algorithm.
      DO STEP=1,RINGSIZE-1
C
        RCVLTH = SIZE(STEP+1)
        RCVDEX = OFFSET(STEP+1)
        SNDLTH = SIZE(STEP)
        SNDDEX = OFFSET(STEP)
C
C       Next, shift vector segment back the other way.
        CALL SR2(COMMOPT, PROTOPT, .FALSE., ORDER, ME, 
     &           STEP+BASE2, LEFT, RBYTES*SNDLTH, SUP(SNDDEX), 
     &           RIGHT, RBYTES*RCVLTH, SUP(RCVDEX))
C
      ENDDO
C
C     Clean up outstanding sends. Note that send buffers are never in
C     danger of being overwritten. During the maximization stage, 
C     send buffer space is never reused. The broadcast stage does
C     not begin until all maximization sends have completed (since the 
C     broadcast data comes from the destination of the maximization
C     sends).
      DO STEP=RINGSIZE-1,1,-1
        RCVLTH = SIZE(STEP)
        RCVDEX = OFFSET(STEP)
        CALL SR3(COMMOPT, PROTOPT, ME, STEP+BASE1, RIGHT,
     &           LEFT, RBYTES*RCVLTH, WS(RCVDEX))
      ENDDO
      DO STEP=1,RINGSIZE-1
        RCVLTH = SIZE(STEP+1)
        RCVDEX = OFFSET(STEP+1)
        CALL SR3(COMMOPT, PROTOPT, ME, STEP+BASE2, LEFT,
     &           RIGHT, RBYTES*RCVLTH, SUP(RCVDEX))
      ENDDO
C
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
