#include "params.h"
      subroutine difcor(doslt   ,klev    ,ztodt   ,delps   ,u       ,
     $                  v       ,qm1     ,pdel    ,pint    ,t       ,
     $                  q       ,tdif    ,qdif    ,udif    ,vdif    )
C-----------------------------------------------------------------------
C
C 1. Add correction term to t and q horizontal diffusions. This term
C provides a partial correction of horizontal diffusion on hybrid (sigma)
C surfaces to horizontal diffusion on pressure surfaces. The appropriate
C function of surface pressure (delps, which already contains the diffusion
C coefficient and the time step) is computed during the transform
C from  spherical harmonic coefficients to grid point values. This term
C can only be applied in the portion of the vertical domain in which
C biharmonic horizontal diffusion is employed. In addition, the term is
C unnecessary on pure pressure levels.
C
C 2. Determine the implied heating rate due to momentum diffusion in order
C to conserve total energy and add it to the temperature.
C
C---------------------------Code history--------------------------------
C
C Original version:  D. Williamson
C Standardized:      J. Rosinski, June 1992
C Reviewed:          B. Boville, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      logical
     $     doslt                 ! true => slt,false => spect transport
      integer
     $     klev                  ! k-index of top hybrid level
      real ztodt,                ! twice time step unless nstep = 0
     $     delps(plond),         ! srf press function for correction
     $     u(plond,plev),        ! u-wind
     $     v(plond,plev),        ! v-wind
     $     qm1(plond,plev),      ! moisture (time n-1)
     $     pdel(plond,plev),     ! pdel(k) = pint(k+1) - pint(k)
     $     pint(plond,plevp)     ! pressure at model interfaces
C
C Input/Output arguments
C
      real t(plond,plev),        ! temperature
     $     q(plond,plev),        ! moisture (time n)
     $     tdif(plond,plev),     ! initial/final temperature diffusion
     $     qdif(plond,plev),     ! initial/final moisture diffusion
     $     udif(plond,plev),     ! initial/final u-momentum diffusion
     $     vdif(plond,plev)      ! initial/final v-momentum diffusion
C
C---------------------------Local workspace-----------------------------
C
      real tcor(plond,plev),     ! temperature correction term
     $     qcor(plond,plev)      ! moisture correction term
      integer
     $     i,k                   ! longitude, level indices
C
C-----------------------------------------------------------------------
C
C Compute the pressure surface correction term for horizontal diffusion of
C temperature. 
C
      do k=klev,plev
         if (k.eq.1) then
            do i=1,plon
               tcor(i,k) = delps(i)*0.5/pdel(i,k)*
     $                      (hybi(k+1)*(t(i,k+1)-t(i,k)))*pint(i,plevp)
            end do
         else if (k.eq.plev) then
            do i=1,plon
               tcor(i,k) = delps(i)*0.5/pdel(i,k)*
     $                      (hybi(k)*(t(i,k)-t(i,k-1)))*pint(i,plevp)
            end do
         else
            do i=1,plon
               tcor(i,k) = delps(i)*0.5/pdel(i,k)*
     $                      (hybi(k+1)*(t(i,k+1)-t(i,k)) +
     $                       hybi(k  )*(t(i,k)-t(i,k-1)))*pint(i,plevp)
            end do
         end if
      end do
C
C Add the temperture diffusion correction to the diffusive heating term 
C and to the temperature.
C
      do k=klev,plev
         do i=1,plon
            tdif(i,k) = tdif(i,k) + tcor(i,k)/ztodt
            t(i,k) = t(i,k) + tcor(i,k)
         end do
      end do
C
C Correct the horizontal diffusion of water if spectral advection is 
C being used. For semi-lagrangian advection, no diffusion is applied, 
C so no correction is required.
C
      if (.not.doslt) then
         do k=klev,plev
            if (k.eq.1) then
               do i=1,plon
                  qcor(i,k) = delps(i)*0.5/pdel(i,k)*
     $                      (hybm(k+1)*(q(i,k+1)-q(i,k)))*pint(i,plevp)
               end do
            else if (k.eq.plev) then
               do i=1,plon
                  qcor(i,k) = delps(i)*0.5/pdel(i,k)*
     $                       (hybm(k)*(q(i,k)-q(i,k-1)))*pint(i,plevp)
               end do
            else
               do i=1,plon
                  qcor(i,k) = delps(i)*0.5/pdel(i,k)*
     $                       (hybm(k+1)*(q(i,k+1)-q(i,k)) +
     $                        hybm(k  )*(q(i,k)-q(i,k-1)))*pint(i,plevp)
               end do
            end if
         end do
         do k=klev,plev
            do i=1,plon
               qdif(i,k) = qdif(i,k) + qcor(i,k)/ztodt
               q(i,k) = q(i,k) + qcor(i,k)
            end do
         end do
      end if
C
C Convert momentum diffusion tendencies to heating rates in order to 
C conserve internal energy. Add the heating to the temperature and to 
C diffusive heating term.
C
      do k=1,plev
         do i=1,plon
            t(i,k) = t(i,k) - ztodt *
     $            (u(i,k)*udif(i,k) + v(i,k)*vdif(i,k)) /
     $                (cpair*(1. + cpvir*qm1(i,k)))
            tdif(i,k) = tdif(i,k) -
     $            (u(i,k)*udif(i,k) + v(i,k)*vdif(i,k)) /
     $                (cpair*(1. + cpvir*qm1(i,k)))
         end do
      end do
C 
      return
      end
