C#######################################################################
C PSTSWM Version 4.0 (12/1/94)                                         #
C  A message-passing benchmark code and parallel algorithm testbed     #
C  that solves the nonlinear shallow water equations using the spectral#
C  transform method.                                                   #
C Written by:                                                          #
C  Patrick Worley of Oak Ridge National Laboratory                     #
C  Ian Foster of Argonne National Laboratory                           #
C Based on the sequential code STSWM 2.0 by James Hack and Ruediger    #
C  Jakob of the National Center for Atmospheric Research.              #
C Research and development funded by the Computer Hardware, Advanced   #
C  Mathematics, and Model Physics (CHAMMP) program of the U.S.         #
C  Department of Energy.                                               # 
C                                                                      #
C Questions and comments should be directed to worley@msr.epm.ornl.gov #
C Please notify and acknowledge the authors in any research or         #
C publications utilizing PSTSWM or any part of the code.               #
C                                                                      #
C NOTICE: Neither the institutions nor the authors make any            #
C representations about the suitability of this software for any       #
C purpose. This software is provided "as is", without express or       #
C implied warranty.                                                    #
C#######################################################################
C include precision declaration definitions                            #
#include "params.h"
C#######################################################################
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C seqfor.F                                                             C
C                                                                      C
C The following sequential routine calculates the inverse block        C
C complex Fourier transform.                                           C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE CFFTB(N, CJUMP, MVECS, TRIG, Y)
C
C SEQFOR calculates in-place (unnormalized) inverse Fourier transforms
C of an array of complex vectors. Each vector is of length N, where N
C is a power of twos, threes, and fives. 
C
C called by: CUBCTB
C calls: FAX235
C
C---- Implicit None ----------------------------------------------------
C
      IMPLICIT NONE
C
C---- Arguments --------------------------------------------------------
C
C     Input
C
C length of complex vector
      INTEGER N
C length of first dimension of data/results array
      INTEGER CJUMP
C number of vectors to be transformed.
      INTEGER MVECS
C trigonometric function values used in the transform
      COMPLEX TRIG(N)
C
C     Input/Output
C
C On input, Y contains the data to be transformed. On output, Y
C contains the results of the transforms.
      COMPLEX Y(CJUMP, MVECS)
C
C---- Local Variables --------------------------------------------------
C
C loop limits for each stage of the forward fast Fourier transform
      INTEGER NSTEPS, NBLOCK, INCREM
C factor of N currently being used in FFT
      INTEGER FACTOR
C index offset used to move through the trigonometric function values
C in the correct order
      INTEGER IC
C index offsets used to access the twiddle factors needed in a given 
C butterfly update
      INTEGER TOFF1, TOFF2, TOFF3, TOFF4, TOFF5
C index offsets used to access the data needed in a given butterfly
C update
      INTEGER YOFF1, YOFF2, YOFF3, YOFF4, YOFF5
C loop indices
      INTEGER I, J, K, L
C temporaries used in computing the forward butterfly updates
      COMPLEX C1, C2, C3, C4, C5
C previous vector length
      INTEGER NSAVE
      SAVE NSAVE
C array containing number of factors and factorization of N
      INTEGER NFAX(0:12), FOURFAX, TWOFAX, THREEFAX, FIVEFAX
      SAVE FOURFAX, TWOFAX, THREEFAX, FIVEFAX
C "local" twiddle factors, used in power-of-three and power-of-five
C butterfly updates
      REAL PI
      COMPLEX A1, A2, B1, B2, B3, B4
      COMPLEX A1C, A2C, B1C, B2C, B3C, B4C
      SAVE A1C, A2C, B1C, B2C, B3C, B4C
C remember whether this is the first time routine called. If yes,
C then compute local values.
      LOGICAL FIRST
      SAVE FIRST
C
C---- Data Statements --------------------------------------------------
C
      DATA NSAVE /0/
      DATA FIRST /.TRUE./
C                                                                             
C---- Executable Statements --------------------------------------------
C
      IF (FIRST) THEN
        PI = 4.0D0*ATAN(1.0D0)
        A1 = EXP(CMPLX(0.0D0,-PI/3.0D0))
        A2 = EXP(CMPLX(0.0D0,-2.0D0*(PI/3.0D0)))
        B1 = EXP(CMPLX(0.0D0,-PI/5.0D0))
        B2 = EXP(CMPLX(0.0D0,-2.0D0*(PI/5.0D0)))
        B3 = EXP(CMPLX(0.0D0,-3.0D0*(PI/5.0D0)))
        B4 = EXP(CMPLX(0.0D0,-4.0D0*(PI/5.0D0)))
        A1C = CONJG(A1)
        A2C = CONJG(A2)
        B1C = CONJG(B1)
        B2C = CONJG(B2)
        B3C = CONJG(B3)
        B4C = CONJG(B4)
        FIRST = .FALSE.
      ENDIF
C
      IF (NSAVE .NE. N) THEN
C       Following logic assumes that all factors are grouped, with
C       4s first, followed by 2s, 3s, and 5s, respectively.
C
        CALL FAX235(N, NFAX)
C
        FOURFAX=0
        TWOFAX=0
        THREEFAX=0
        FIVEFAX=0
        DO K=1,NFAX(0)
          IF (NFAX(K) .EQ. 4) THEN
            FOURFAX = FOURFAX + 1
          ELSEIF (NFAX(K) .EQ. 2) THEN
            TWOFAX = TWOFAX + 1
          ELSEIF (NFAX(K) .EQ. 3) THEN
            THREEFAX = THREEFAX + 1
          ELSEIF (NFAX(K) .EQ. 5) THEN
            FIVEFAX = FIVEFAX + 1
          ELSE
            WRITE(6,*) 'CFFTB: N contains factors other than 2,3,4,5.'
            STOP
          ENDIF
        ENDDO
C
        NSAVE = N
      ENDIF
C
      DO L=1,MVECS
C
        INCREM = 1
        IC     = N-1
        NBLOCK = N
C
        DO K=1,FIVEFAX
C
          IC = IC - 4*INCREM
          NBLOCK  = NBLOCK/5
C
          TOFF2 = IC
          TOFF3 = TOFF2 + INCREM
          TOFF4 = TOFF3 + INCREM
          TOFF5 = TOFF4 + INCREM
          DO J=1,NBLOCK
C
            YOFF1 = 5*(J-1)*INCREM
            YOFF2 = YOFF1 + INCREM
            YOFF3 = YOFF2 + INCREM
            YOFF4 = YOFF3 + INCREM
            YOFF5 = YOFF4 + INCREM
            DO I=1,INCREM
C
C             Calculate power-of-five butterfly update.
              C2 = Y(YOFF2+I,L)*TRIG(TOFF2+I)
              C3 = Y(YOFF3+I,L)*TRIG(TOFF3+I)
              C4 = Y(YOFF4+I,L)*TRIG(TOFF4+I)
              C5 = Y(YOFF5+I,L)*TRIG(TOFF5+I)
              Y(YOFF2+I,L) = Y(YOFF1+I,L) 
     &                     + B2C*C2 + B4C*C3 - B1C*C4 - B3C*C5
              Y(YOFF3+I,L) = Y(YOFF1+I,L) 
     &                     + B4C*C2 - B3C*C3 + B2C*C4 - B1C*C5
              Y(YOFF4+I,L) = Y(YOFF1+I,L) 
     &                     - B1C*C2 + B2C*C3 - B3C*C4 + B4C*C5
              Y(YOFF5+I,L) = Y(YOFF1+I,L) 
     &                     - B3C*C2 - B1C*C3 + B4C*C4 + B2C*C5
              Y(YOFF1+I,L) = Y(YOFF1+I,L) + C2 + C3 + C4 + C5
C
            ENDDO
C
          ENDDO
C
          INCREM = INCREM*5
C
        ENDDO
C
        DO K=1,THREEFAX
C
          IC = IC - 2*INCREM
          NBLOCK  = NBLOCK/3
C
          TOFF2 = IC
          TOFF3 = TOFF2 + INCREM
          DO J=1,NBLOCK
C
            YOFF1 = 3*(J-1)*INCREM
            YOFF2 = YOFF1 + INCREM
            YOFF3 = YOFF2 + INCREM
            DO I=1,INCREM
C
C             Calculate power-of-three butterfly update.
              C2 = Y(YOFF2+I,L)*TRIG(TOFF2+I)
              C3 = Y(YOFF3+I,L)*TRIG(TOFF3+I)
              Y(YOFF2+I,L) = Y(YOFF1+I,L) + A2C*C2 - A1C*C3
              Y(YOFF3+I,L) = Y(YOFF1+I,L) - A1C*C2 + A2C*C3
              Y(YOFF1+I,L) = Y(YOFF1+I,L) + C2 + C3
C
            ENDDO
C
          ENDDO
C
          INCREM = INCREM*3
C
        ENDDO
C
        DO K=1,TWOFAX
C
          IC = IC - INCREM
          NBLOCK  = NBLOCK/2
C
          TOFF2 = IC
          DO J=1,NBLOCK
C
            YOFF1 = 2*(J-1)*INCREM
            YOFF2 = YOFF1 + INCREM
            DO I=1,INCREM
C
C             Calculate power-of-two butterfly update.
              C2 = Y(YOFF2+I,L)*TRIG(TOFF2+I)
              Y(YOFF2+I,L) = Y(YOFF1+I,L) - C2
              Y(YOFF1+I,L) = Y(YOFF1+I,L) + C2
C
            ENDDO
C
          ENDDO
C
          INCREM = INCREM*2
C
        ENDDO
C
        DO K=1,FOURFAX
C
          IC = IC - 3*INCREM
          NBLOCK  = NBLOCK/4
C
          TOFF2 = IC
          TOFF3 = TOFF2 + INCREM
          TOFF4 = TOFF3 + INCREM
          DO J=1,NBLOCK
C
            YOFF1 = 4*(J-1)*INCREM
            YOFF2 = YOFF1 + INCREM
            YOFF3 = YOFF2 + INCREM
            YOFF4 = YOFF3 + INCREM
            DO I=1,INCREM
C
C             Calculate power-of-four butterfly update.
              C2 = Y(YOFF2+I,L)*TRIG(TOFF2+I)
              C3 = Y(YOFF3+I,L)*TRIG(TOFF3+I)
              C4 = Y(YOFF4+I,L)*TRIG(TOFF4+I)
C
C Y(YOFF2+I,L) = (Y(YOFF1+I,L) - C3) - CMPLX(0.0,-1.0)*(C2-C4)
C Y(YOFF4+I,L) = (Y(YOFF1+I,L) - C3) + CMPLX(0.0,-1.0)*(C2-C4)
              Y(YOFF2+I,L) = (Y(YOFF1+I,L) - C3)
     &                     - (CMPLX(AIMAG(C2)-AIMAG(C4),
     &                              AREAL(C4)-AREAL(C2)))
              Y(YOFF4+I,L) = (Y(YOFF1+I,L) - C3)
     &                     + (CMPLX(AIMAG(C2)-AIMAG(C4),
     &                              AREAL(C4)-AREAL(C2)))
              Y(YOFF3+I,L) = (Y(YOFF1+I,L) + C3) - (C2+C4)
              Y(YOFF1+I,L) = (Y(YOFF1+I,L) + C3) + (C2+C4)
C
            ENDDO
C
          ENDDO
C
          INCREM = INCREM*4
C
        ENDDO
C
      ENDDO
C
      RETURN
      END
